package com.dragonsoft.dcuc.approvegateway.business;

import cn.hutool.core.util.StrUtil;
import cn.hutool.http.HttpRequest;
import com.dragonsoft.dcuc.approvegateway.pojo.*;
import com.dragonsoft.dcuc.approvegateway.properties.DcucApproveProperties;
import com.dragonsoft.duceap.commons.util.UUIDUtils;
import com.dragonsoft.duceap.commons.util.json.JsonUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * <p>
 * 竹云认证相关业务
 * </p>
 *
 * @author huangzqa
 * @date 2021/6/21
 */
@Slf4j
@Service
public class BimBusiness {

    @Autowired
    private DcucApproveProperties dcucApproveProperties;

    /**
     * 获取 token
     *
     * @return token
     */
    public String getToken() {
        String url = dcucApproveProperties.getBimIdentityUrl() + "/api/rest/management/ExtApiMgmtAuthService/login";

        BimLoginReqVO bimLoginReqVo = new BimLoginReqVO();
        bimLoginReqVo.setLoginId(dcucApproveProperties.getBimLoginId());
        bimLoginReqVo.setSecretKey(dcucApproveProperties.getBimSecretKey());

        String requestJson = JsonUtils.toJSONString(bimLoginReqVo);
        log.debug("Bim login request:{}", requestJson);
        String respJson = HttpRequest.post(url)
                .body(requestJson)
                .execute()
                .body();
        log.debug("Bim login respJson:{}", respJson);

        BimRespVO bimRespVo = JsonUtils.parseObject(respJson, BimRespVO.class);
        if (!bimRespVo.getSuccess()) {
            log.error("Bim login token error :{}", respJson);
            return "";
        }

        Object data = bimRespVo.getData();
        return (String) data;
    }

    /**
     * 退出 token
     *
     * @param token token
     */
    public void logoutToken(String token) {

        String url = dcucApproveProperties.getBimIdentityUrl() + "/api/rest/management/ExtApiMgmtAuthService/logout";

        BimLogoutReqVO bimLogoutReqVo = new BimLogoutReqVO();
        bimLogoutReqVo.setToken(token);

        String requestJson = JsonUtils.toJSONString(bimLogoutReqVo);
        log.debug("Bim logout requestJson:{}", requestJson);
        String respJson = HttpRequest.post(url)
                .body(requestJson)
                .execute()
                .body();
        log.debug("Bim logout respJson:{}", respJson);

        BimRespVO bimRespVo = JsonUtils.parseObject(respJson, BimRespVO.class);
        if (!bimRespVo.getSuccess()) {
            log.error("Bim logout token error :{}", respJson);
        }
    }


    /**
     * 获取用户信息
     *
     * @param userToken 用户令牌
     * @param token     token
     * @return 用户信息
     */
    public BimUserInfoItemRespVO getUserInfoByUserToken(String userToken, String token) {

        if (StrUtil.isBlank(userToken)) {
            throw new IllegalArgumentException("UserToken is blank.");
        }

        String messageId = UUIDUtils.getUUID();

        BimUserInfoReqVO bimUserInfoReqVO = new BimUserInfoReqVO();
        bimUserInfoReqVO.setToken(token);
        bimUserInfoReqVO.setMessageId(messageId);
        bimUserInfoReqVO.setUserToken(userToken);

        String reqJson = JsonUtils.toJSONString(bimUserInfoReqVO);

        log.info("UserInfo req :{}", reqJson);

        String url = dcucApproveProperties.getBimIdentityUrl() + "/api/rest/customization/ExpApiCustomDragonitService/getUserInfoByUserToken";

        log.info("UserInfo req url:{}", reqJson);
        String postResp = HttpRequest.post(url)
                .body(reqJson)
                .execute()
                .body();
        log.info("UserInfo resp :{}", postResp);

        BimUserInfoRespVO bimRespVo = JsonUtils.parseObject(postResp, BimUserInfoRespVO.class);

        if (!bimRespVo.getSuccess()) {
            log.error("UserInfo error:{}", postResp);
        }

        return bimRespVo.getData();
    }
}
