package com.dragonsoft.dcuc.approvegateway.filter;

import com.alibaba.fastjson.JSONObject;
import com.dragonsoft.duceap.base.entity.security.BaseSecurityUser;
import com.dragonsoft.duceap.base.utils.UserContextUtils;
import com.dragonsoft.duceap.commons.util.ip.IpUtils;
import com.dragonsoft.duceap.security.jwt.JwtTokenUtils;
import com.netflix.zuul.ZuulFilter;
import com.netflix.zuul.context.RequestContext;
import com.netflix.zuul.exception.ZuulException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.netflix.zuul.filters.support.FilterConstants;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletRequest;
import java.util.Map;

/**
 * <p>
 * 获取请求者真实IP
 * </p>
 *
 * @author huangzqa
 * @date 2021/6/29
 */
@Slf4j
@Component
public class UserInfoForwardForFilter extends ZuulFilter {
    @Override
    public String filterType() {
        return FilterConstants.PRE_TYPE;
    }

    @Override
    public int filterOrder() {
        return 0;
    }

    @Override
    public boolean shouldFilter() {
        return true;
    }

    @Override
    public Object run() throws ZuulException {
        RequestContext currentContext = RequestContext.getCurrentContext();
        HttpServletRequest request = currentContext.getRequest();
        String realIpAdrress = IpUtils.getRealIpAdrress(request);
        Map<String, String> zuulRequestHeaders = currentContext.getZuulRequestHeaders();
        zuulRequestHeaders.put("HTTP_X_FORWARDED_FOR", realIpAdrress);
        BaseSecurityUser currentUser = UserContextUtils.getCurrentUser();
        log.info("====登录用户信息：{}====", JSONObject.toJSONString(currentUser));
        if (currentUser != null) {
            String jwtToken = JwtTokenUtils.getAlgorithmGen(JwtTokenUtils.AlgorithmType.HS256).sign(currentUser);
            currentContext.addZuulRequestHeader(JwtTokenUtils.AUTHORIZATION_HEADER, JwtTokenUtils.TOKEN_PREFIX + jwtToken);
            log.info("登录jwtToken：{}", jwtToken);
        }
        return null;
    }
}
